
require 'racc/parser'
#
# confp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/confs'

module EST

class ConfigFileParser < Racc::Parser

  class << self
    def parse( fn )
      new.parse( fn )
    end
  end

  def initialize( dflag = false )
    @yydebug = dflag
  end

  def parse( fn )
    unless File.exist? fn then
      raise ArgumentError, "config file #{fn} not exist"
    end
    @fname = fn
    str = nil
    File.open( fn ) {|f| str = f.read }
    @scanner = ConfigFileScanner.new( str )

    do_parse
  end


  private

  def next_token
    @scanner.scan
  end

  def on_error( et, ev, _values )
    raise ParseError,
      "\nEST rc file syntax error: file #{@fname}, line #{@scanner.lineno}"
  end


  def set_value0( hash )
    h = @hstack[-1]
    h.update hash
  end
    
  def set_value1( key, val )
    h = @hstack[-1]
    h[ key ] = val
  end

  def set_value2( key1, key2, val )
    h = @hstack[-1]
    if tmp = h[ key1 ] then
      tmp[ key2 ] = val
    else
      h[ key1 ] = { key2 => val }
    end
  end

##### racc 1.0.2 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 0, 14, :_reduce_1,
 2, 12, :_reduce_2,
 1, 13, :_reduce_none,
 3, 13, :_reduce_none,
 0, 15, :_reduce_none,
 2, 15, :_reduce_6,
 3, 15, :_reduce_7,
 2, 15, :_reduce_8,
 3, 15, :_reduce_9,
 4, 15, :_reduce_10,
 1, 16, :_reduce_none,
 1, 16, :_reduce_none,
 1, 19, :_reduce_none,
 1, 19, :_reduce_none,
 0, 22, :_reduce_15,
 4, 20, :_reduce_16,
 3, 21, :_reduce_17,
 0, 23, :_reduce_18,
 3, 23, :_reduce_19,
 0, 24, :_reduce_none,
 1, 24, :_reduce_none,
 1, 24, :_reduce_none,
 1, 17, :_reduce_none,
 2, 17, :_reduce_24,
 1, 18, :_reduce_none,
 1, 18, :_reduce_none ]

racc_reduce_n = 27

racc_shift_n = 39

racc_action_table = [
    30,    21,     5,     6,     7,    27,    21,    38,    31,    15,
    34,    11,    26,     7,    16,    15,    18,    11,    19,    20,
     5,     6,     7,     5,     6,     7,    15,    10,    11,    26,
     7,    26,     7,    26,     7,     3 ]

racc_action_check = [
    22,    32,     2,     2,     2,    19,     8,    32,    22,    19,
    27,    19,    30,    30,     5,    30,     6,    30,     6,     6,
    21,    21,    21,    23,    23,    23,     4,     3,     4,    24,
    24,    17,    17,    36,    36,     1 ]

racc_action_default = [
    -1,   -27,    -5,   -27,   -27,   -27,   -25,   -26,    -2,    -3,
    39,   -18,    -8,   -13,   -14,   -15,    -6,   -27,   -12,   -27,
   -11,    -5,   -27,    -5,    -7,   -23,   -25,   -27,    -9,    -4,
   -20,   -17,   -27,   -24,   -10,   -19,   -21,   -22,   -16 ]

racc_action_pointer = [
   nil,    35,    -1,    27,    19,    10,    13,   nil,     4,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   nil,    27,   nil,     2,
   nil,    17,    -2,    20,    25,   nil,   nil,     6,   nil,   nil,
     8,   nil,    -1,   nil,   nil,   nil,    29,   nil,   nil ]

racc_goto_table = [
    12,     8,     1,    24,    25,    17,    29,     2,    23,    22,
    35,    33,   nil,   nil,   nil,    28,    36,    25,   nil,   nil,
   nil,   nil,    32,    33,   nil,   nil,    37 ]

racc_goto_check = [
     8,     2,     1,     6,     7,     5,     4,     3,    11,    12,
    13,     7,   nil,   nil,   nil,     8,     6,     7,   nil,   nil,
   nil,   nil,     2,     7,   nil,   nil,     8 ]

racc_goto_pointer = [
   nil,     2,    -1,     7,   -15,    -1,   -14,   -13,    -4,   nil,
   nil,    -7,    -2,   -20 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,     9,   nil,   nil,     4,   nil,    13,
    14,   nil,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :EOL => 2,
 "=" => 3,
 :STRING => 4,
 :QUOTED => 5,
 ":" => 6,
 "{" => 7,
 "}" => 8,
 "[" => 9,
 "]" => 10 }

racc_nt_base = 11

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n ]

Racc_debug_parser = false

##### racc system variables end #####


 # reduce 0 omitted

 module_eval( <<'.,.,', 'rr.y', 12 )
  def _reduce_1( val, _values, result )
                    @hstack = [ {} ]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 16 )
  def _reduce_2( val, _values, result )
                    result = @hstack[0]
   result
  end
.,.,

 # reduce 3 omitted

 # reduce 4 omitted

 # reduce 5 omitted

 module_eval( <<'.,.,', 'rr.y', 27 )
  def _reduce_6( val, _values, result )
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[1], 'rc' ) )
                    set_value0 hash
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 33 )
  def _reduce_7( val, _values, result )
                    set_value1 result, val[2]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 37 )
  def _reduce_8( val, _values, result )
                    set_value1 result, val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 41 )
  def _reduce_9( val, _values, result )
                    set_value2 result, val[1], val[2]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 45 )
  def _reduce_10( val, _values, result )
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[3], 'rc' ) )
                    set_value2 result, val[1], hash
   result
  end
.,.,

 # reduce 11 omitted

 # reduce 12 omitted

 # reduce 13 omitted

 # reduce 14 omitted

 module_eval( <<'.,.,', 'rr.y', 59 )
  def _reduce_15( val, _values, result )
                    @hstack.push Hash.new
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 63 )
  def _reduce_16( val, _values, result )
                    result = @hstack.pop
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 69 )
  def _reduce_17( val, _values, result )
                    result = val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 75 )
  def _reduce_18( val, _values, result )
                    result = []
   result
  end
.,.,

 module_eval( <<'.,.,', 'rr.y', 79 )
  def _reduce_19( val, _values, result )
                    result.push val[2] if val[2]
   result
  end
.,.,

 # reduce 20 omitted

 # reduce 21 omitted

 # reduce 22 omitted

 # reduce 23 omitted

 module_eval( <<'.,.,', 'rr.y', 91 )
  def _reduce_24( val, _values, result )
                    result = [result] unless Array === result
                    result.push val[1]
   result
  end
.,.,

 # reduce 25 omitted

 # reduce 26 omitted

end

end   # module EST
