#
# estcore.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'amstd/must'
require 'amstd/bug'
require 'amstd/fileutils'

def dputs( msg )
  puts msg if $DEBUG
end
def dp( msg )
  p msg if $DEBUG
end

module EST

  class ESTExit < Exception; end


  def EST.create( rcdir )
    const_set :CORE, Core.new( rcdir )
  end

  class CCore

    def initialize( usrdir )
      @corerc = {}
      initialize_core_rc()
      @corerc['usrdir'] = usrdir
      @corerc['rcdir'] = usrdir + '/rc'

      @mailbox  = Mailset.new
      @sender   = SenderManager.new
      @popper   = PopperManager.new
      @composer = ComposerManager.new
      @getter   = GetterManager.new
      @refiler  = RefilerManager.new
      @profile  = ProfileManager.new
      @address  = AddressManager.new

      @addrbook = nil
    end


    attr :mailbox
    attr :sender
    attr :popper
    attr :composer
    attr :getter
    attr :refiler
    attr :profile
    attr :address


    private


    require 'est/estconf'   # def initialize_core_rc()

    def construct
      ensure_config
      first_profile = RcReader.read( search_file( 'config', 'rc' ) )
      first_profile
    end

    def ensure_config
      rcdir = @corerc['rcdir']
      unless File.exist? rcdir then
        # FileUtils.mkdir_p rcdir
bug! rcdir
      end
      unless File.directory? rcdir then
        raise ArgumentError, "config directory '#{rcdir}' is not directory"
      end
      cf = File.join( rcdir, 'config' )
      unless File.exist? cf then
bug! "config file #{cf} does not exist"
        FILEUTILS.touch cf
      else
        unless File.file? cf then
bug! cf
          raise ArgumentError, "config file '#{cf}'is not file"
        end
      end
    end


    public


    def shutdown
dputs 'EST::CORE.shutdown'
      @mailbox.shutdown if @mailbox
dputs 'done'
    end


    def notice( str )
      $stderr.puts str
    end

    def warn( str )
      $stderr.puts str
    end
    
    def []( key )
      @corerc[key]
    end


    def search_file( fname, addpath = '' )
      %w( usrdir libdir ).each do |loc|
        path = File.join( @corerc[ loc ], addpath, fname )
        return path if File.exist? path
      end

      raise ArgumentError,
        "could not find file '#{File.join addpath, fname}'"
    end


    def mvmail( from, to, idx )
      return unless from = to_mbox_from( from )
      return unless to   = to_mbox_to( to )
      return if from == to
      return unless from[idx]

      to.move_of from, idx
    end

    def rmmail( mb, idx )
      return unless mbox = to_mbox_from( mb )
      mbox.remove idx
    end

    def mkmail( mb = nil, str = nil )
      if mb then
        mbox = to_mbox_to( mb )
        return unless mbox
      else
        mbox = mailbox.draft
      end

      port = mbox.new
      port.wopen do |wp|
        if iterator? then
          yield wp
        elsif str then
          wp.write str
        else
          raise ArgumentError, "mkmail: no argument"
        end
      end
      mbox.add port

      return port
    end

    def to_mbox_to( to )
      case to
      when Mailbox then to
      when String  then @mailbox[ to ]
      else
        to.must Mailbox, String
      end
    end

    def to_mbox_from( from )
      case from
      when Mailbox then from
      when String  then @mailbox[ from ]
      else
        to.must Mailbox, String
      end
    end


    def new_mail
      st = profile.get_current( 'composer/new-mail' )
      edit st.compose
    end

    def reply_to( orig )
      unless orig then
        warn 'nil for CORE.reply_to'
        return
      end
      st = profile.get_current( 'composer/reply-mail' )
      edit st.compose( orig )
    end

    def forward( orig )
      unless orig then
        warn 'nil for CORE.forward'
        return
      end
      st = profile.get_current( 'composer/forward-mail' )
      edit st.compose( orig )
    end


    def current_sender
      profile.get_current('sender')
    end

    def send_all_draft
      current_sender.exec
    end

    def send_from_draft( *idxes )
      current_sender.exec( *idxes )
    end

    def send_given
      current_sender.exec do |sender|
        yield sender
      end
    end


    def current_popper
      profile.get_current('popper')
    end

    def pop_all
      current_popper.exec
    end


    def current_refiler
      profile.get_current('refiler')
    end

    def refile
dputs 'doing refile'
      if c = current_refiler then
dputs 'refiler exist'
        c.refile
dputs 'done'
      end
    end

  end

end   # module EST
